function 当量煤耗变化量(热耗_kJ_kWh, 功率_MW, 冷端设备耗功变化_kW) {
    /**
     * 计算多开一台循泵或风机时，供电煤耗的最大增大值，不适用于少开一台循泵或风机
     * @param 功率_MW: 冷端设备运行方式变化前发电机出口功率
     */
    let hr = 热耗_kJ_kWh;
    let power1 = 功率_MW; // 冷端设备运行方式变化前发电机出口功率
    let power2 = power1 + 冷端设备耗功变化_kW / 1000; // 冷端设备运行方式变化后发电机出口功率，需要该功率才能保证供电功率不变
    let 燃料热量 = hr * power1 * 1000;  // kJ/h
    let 燃料热量2 = hr * power2 * 1000;
    let 标煤热值 = 29260; // kJ/kg
    let 耗煤量1 = 燃料热量 / 标煤热值;
    let 耗煤量2 = 燃料热量2 / 标煤热值;
    let 供电煤耗1 = 耗煤量1 / power1;
    let 供电煤耗2 = 耗煤量2 / power1;
    return 供电煤耗2 - 供电煤耗1;
}