# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from yangke.common.QtImporter import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.content_tab = QtWidgets.QTabWidget(self.splitter)
        self.content_tab.setToolTip("")
        self.content_tab.setElideMode(QtCore.Qt.ElideNone)
        self.content_tab.setDocumentMode(False)
        self.content_tab.setTabsClosable(True)
        self.content_tab.setMovable(True)
        self.content_tab.setTabBarAutoHide(False)
        self.content_tab.setObjectName("content_tab")
        self.verticalLayout_5.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.centralwidget)
        self.dockWidget_ATW = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_ATW.setObjectName("dockWidget_ATW")
        self.dockWidgetContents_3 = QtWidgets.QWidget()
        self.dockWidgetContents_3.setObjectName("dockWidgetContents_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter_3 = QtWidgets.QSplitter(self.dockWidgetContents_3)
        self.splitter_3.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_3.setObjectName("splitter_3")
        self.ActiveToolWindow = QtWidgets.QTabWidget(self.splitter_3)
        self.ActiveToolWindow.setTabPosition(QtWidgets.QTabWidget.South)
        self.ActiveToolWindow.setObjectName("ActiveToolWindow")
        self.Terminal1 = QtWidgets.QWidget()
        self.Terminal1.setObjectName("Terminal1")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.Terminal1)
        self.verticalLayout_11.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_11.setSpacing(1)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.splitter_2 = QtWidgets.QSplitter(self.Terminal1)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName("splitter_2")
        self.Terminal = QtWidgets.QWidget(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Terminal.sizePolicy().hasHeightForWidth())
        self.Terminal.setSizePolicy(sizePolicy)
        self.Terminal.setMinimumSize(QtCore.QSize(200, 20))
        self.Terminal.setStyleSheet("background-color: rgb(85, 255, 255);")
        self.Terminal.setObjectName("Terminal")
        self.TerminalVar = QtWidgets.QWidget(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.TerminalVar.sizePolicy().hasHeightForWidth())
        self.TerminalVar.setSizePolicy(sizePolicy)
        self.TerminalVar.setMinimumSize(QtCore.QSize(200, 20))
        self.TerminalVar.setStyleSheet("background-color: rgb(255, 170, 255);")
        self.TerminalVar.setObjectName("TerminalVar")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.TerminalVar)
        self.verticalLayout_12.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_12.setSpacing(1)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.verticalLayout_11.addWidget(self.splitter_2)
        self.ActiveToolWindow.addTab(self.Terminal1, "")
        self.Log = QtWidgets.QWidget()
        self.Log.setObjectName("Log")
        self.ActiveToolWindow.addTab(self.Log, "")
        self.verticalLayout_2.addWidget(self.splitter_3)
        self.dockWidget_ATW.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockWidget_ATW)
        self.dockWidget_4 = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_4.setFeatures(QtWidgets.QDockWidget.DockWidgetClosable|QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget_4.setObjectName("dockWidget_4")
        self.dockWidgetContents_7 = QtWidgets.QWidget()
        self.dockWidgetContents_7.setObjectName("dockWidgetContents_7")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.dockWidgetContents_7)
        self.verticalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_4.setSpacing(1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.navi_tab = QtWidgets.QTabWidget(self.dockWidgetContents_7)
        self.navi_tab.setObjectName("navi_tab")
        self.component_tab = QtWidgets.QWidget()
        self.component_tab.setObjectName("component_tab")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.component_tab)
        self.verticalLayout_7.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_7.setSpacing(1)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.lv_com = QtWidgets.QListView(self.component_tab)
        self.lv_com.setObjectName("lv_com")
        self.verticalLayout_7.addWidget(self.lv_com)
        self.navi_tab.addTab(self.component_tab, "")
        self.file_tab = QtWidgets.QWidget()
        self.file_tab.setObjectName("file_tab")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.file_tab)
        self.verticalLayout_9.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_9.setSpacing(1)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tv_file = QtWidgets.QTreeView(self.file_tab)
        self.tv_file.setObjectName("tv_file")
        self.verticalLayout_9.addWidget(self.tv_file)
        self.navi_tab.addTab(self.file_tab, "")
        self.scene_tab = QtWidgets.QWidget()
        self.scene_tab.setObjectName("scene_tab")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.scene_tab)
        self.verticalLayout_8.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_8.setSpacing(1)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.lv_scene = QtWidgets.QListView(self.scene_tab)
        self.lv_scene.setObjectName("lv_scene")
        self.verticalLayout_8.addWidget(self.lv_scene)
        self.navi_tab.addTab(self.scene_tab, "")
        self.verticalLayout_4.addWidget(self.navi_tab)
        self.dockWidget_4.setWidget(self.dockWidgetContents_7)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget_4)
        self.dockWidget_5 = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_5.setMinimumSize(QtCore.QSize(300, 400))
        self.dockWidget_5.setObjectName("dockWidget_5")
        self.dockWidgetContents_8 = QtWidgets.QWidget()
        self.dockWidgetContents_8.setObjectName("dockWidgetContents_8")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents_8)
        self.verticalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.detail_tab = QtWidgets.QTabWidget(self.dockWidgetContents_8)
        self.detail_tab.setObjectName("detail_tab")
        self.layout_detail_tab = QtWidgets.QWidget()
        self.layout_detail_tab.setObjectName("layout_detail_tab")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.layout_detail_tab)
        self.verticalLayout_6.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_6.setSpacing(1)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.info_tab = QtWidgets.QTabWidget(self.layout_detail_tab)
        self.info_tab.setStyleSheet("")
        self.info_tab.setTabPosition(QtWidgets.QTabWidget.South)
        self.info_tab.setObjectName("info_tab")
        self.sfv1_2 = QtWidgets.QWidget()
        self.sfv1_2.setObjectName("sfv1_2")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.sfv1_2)
        self.verticalLayout_10.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_10.setSpacing(1)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.scrollArea = QtWidgets.QScrollArea(self.sfv1_2)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.sfv = QtWidgets.QWidget()
        self.sfv.setGeometry(QtCore.QRect(0, 0, 278, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sfv.sizePolicy().hasHeightForWidth())
        self.sfv.setSizePolicy(sizePolicy)
        self.sfv.setObjectName("sfv")
        self.scrollArea.setWidget(self.sfv)
        self.verticalLayout_10.addWidget(self.scrollArea)
        self.info_tab.addTab(self.sfv1_2, "")
        self.result = QtWidgets.QWidget()
        self.result.setObjectName("result")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.result)
        self.verticalLayout_13.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_13.setSpacing(0)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.result)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea_2.sizePolicy().hasHeightForWidth())
        self.scrollArea_2.setSizePolicy(sizePolicy)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.res = QtWidgets.QWidget()
        self.res.setGeometry(QtCore.QRect(0, 0, 280, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.res.sizePolicy().hasHeightForWidth())
        self.res.setSizePolicy(sizePolicy)
        self.res.setObjectName("res")
        self.scrollArea_2.setWidget(self.res)
        self.verticalLayout_13.addWidget(self.scrollArea_2)
        self.info_tab.addTab(self.result, "")
        self.material = QtWidgets.QWidget()
        self.material.setObjectName("material")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.material)
        self.verticalLayout_14.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_14.setSpacing(0)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.scrollArea_3 = QtWidgets.QScrollArea(self.material)
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollArea_3.setObjectName("scrollArea_3")
        self.mat = QtWidgets.QWidget()
        self.mat.setGeometry(QtCore.QRect(0, 0, 280, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mat.sizePolicy().hasHeightForWidth())
        self.mat.setSizePolicy(sizePolicy)
        self.mat.setObjectName("mat")
        self.scrollArea_3.setWidget(self.mat)
        self.verticalLayout_14.addWidget(self.scrollArea_3)
        self.info_tab.addTab(self.material, "")
        self.verticalLayout_6.addWidget(self.info_tab)
        self.detail_tab.addTab(self.layout_detail_tab, "")
        self.verticalLayout.addWidget(self.detail_tab)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_set_value = QtWidgets.QPushButton(self.dockWidgetContents_8)
        self.btn_set_value.setObjectName("btn_set_value")
        self.horizontalLayout.addWidget(self.btn_set_value)
        self.btn_cancel = QtWidgets.QPushButton(self.dockWidgetContents_8)
        self.btn_cancel.setObjectName("btn_cancel")
        self.horizontalLayout.addWidget(self.btn_cancel)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        self.dockWidget_5.setWidget(self.dockWidgetContents_8)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget_5)

        self.retranslateUi(MainWindow)
        self.content_tab.setCurrentIndex(-1)
        self.ActiveToolWindow.setCurrentIndex(0)
        self.navi_tab.setCurrentIndex(0)
        self.detail_tab.setCurrentIndex(0)
        self.info_tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

        self.content_tab.tabCloseRequested

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.ActiveToolWindow.setTabText(self.ActiveToolWindow.indexOf(self.Terminal1), _translate("MainWindow", "终端"))
        self.ActiveToolWindow.setTabText(self.ActiveToolWindow.indexOf(self.Log), _translate("MainWindow", "日志"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.component_tab), _translate("MainWindow", "组件"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.file_tab), _translate("MainWindow", "目录"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.scene_tab), _translate("MainWindow", "场景"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.sfv1_2), _translate("MainWindow", "规格值"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.result), _translate("MainWindow", "结果"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.material), _translate("MainWindow", "材料"))
        self.detail_tab.setTabText(self.detail_tab.indexOf(self.layout_detail_tab), _translate("MainWindow", "组件信息"))
        self.btn_set_value.setText(_translate("MainWindow", "确定"))
        self.btn_cancel.setText(_translate("MainWindow", "取消"))
