"""
向Word页面中添加水印的方法：
1.从已经编号的PDF报告中，抠出背景透明的水印.png图片
    1.1 使用WPS打开PDF文件，提取水印与文字重叠最少的一页至新的PDF文件中。
    1.2 使用GIMP2.0打开提取的PDF，分辨率设置为300。另存为“水印.jpg”文件
    1.3 使用GIMP2.0打开“水印.jpg"文件，删除除水印外的其他文字或图标等页面元素，然后选择水印文字区，以纯黑色填充水印选取，然后导出为”水印.png"文件。
    1.4 使用GIMP2.0打开“水印.png"文件，添加透明通道后，使用颜色选择工具扣出水印文字，并删除背景，使背景为透明，并保存。
    注意：以上操作过程中，不改变图片大小，不进行裁剪。
2.将水印图片加入修改后的WORD页面中生成对应页面的PDF文件。
    2.1 将修改后的WORD页面转换为PDF文件，命名为1.pdf。
    2.2 使用WPS打开1.pdf，选择【编辑】-【水印】-【添加】/【更新水印】（第一次为添加，后续为更新），打开水印编辑界面。
    2.3 在水印编辑界面中，选择水印来源为”水印.png"，旋转为"水平"，不透明度为"18%"，相对页面比例为"100%"，垂直对齐为0，水平对齐为"居中2mm",
        然后确定即可。
    2.4
3. 将修改后的PDF页面替换至原PDF文件中。
    3.1 使用WPS打开原PDF文件，选择【页面】-【替换页面】，在弹出窗口中选择第2步生成的PDF即可。
"""