"""
人脸识别

1. 多张人脸识别 √

2.特征识别，年龄、性别、     闭眼、胡须、口罩、眼镜
    需要进行模型训练，训练后可以满足

3.活体检测
  防止静态照片识别  √
  防止录播视频识别
    最低要求需要输入视频文件
    可以提供特定场景识别，需要进行模型训练
    通用场景，3D人脸识别，最好配备主动近红外检测、3D结构光、TOF、双目立体视觉中的一种或多种

4.多角度人脸识别
  正面  √
  侧面  √
  后面  后面一般无法识别人脸，如果需要检测人，可以想其他办法

5.人脸查询检索匹配
  1：1  √
  1：N  √

目前主要是精度问题，可以满足一般需求
"""
import sys

# sys.path.append("..")
